unit About;

interface

uses
	Windows, SysUtils, Forms, Classes, Controls, StdCtrls, ExtCtrls, Graphics;

{
(* ProgressBarnNX *)
type
	TProgress = class
    private
		FCanvas		: TCanvas;
		FRealRect 	,
        FDrawRect	: TRect;
		FMaxValue	: double;
        FValue		: double;
        FForeColor	,
        FBackColor	: TColor;
        FBitmap		: TBitmap;
        procedure	SetValue(AValue: double);
		procedure	Draw();
    public
		constructor	Create(ACanvas: TCanvas; ARect: TRect; AMaxValue: double);
        destructor	Destroy; override;
    protected

    published
		property	ForeColor: TColor read FForeColor write FForeColor;
		property	BackColor: TColor read FBackColor write FBackColor;
		property	Value: double read FValue write SetValue;
    end;
}
type
  TAboutBox = class(TForm)
    pnlHeader: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    LblURL: TLabel;
    Image1: TImage;
    Panel1: TPanel;
    lblCopy: TLabel;
    Label5: TLabel;
    procedure LblURLClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
  private
    { Private 錾 }
  public
    { Public 錾 }
  end;

var
  AboutBox: TAboutBox;

implementation

uses
	iUtils, Math;

{$R *.dfm}

//------------------------------------------------------------------------------
{
(* RXgN^ *)
constructor	TProgress.Create(ACanvas: TCanvas; ARect: TRect; AMaxValue: double);
begin
	inherited Create;
    FCanvas   := ACanvas;
    FRealRect := ARect;
	with FRealRect do FDrawRect := Rect(0, 0, Right - Left, Bottom - Top);
    FMaxValue := AMaxValue;
    FForeColor := clBlue;
    FBackColor := clRed;
	FBitmap   := TBitmap.Create;
    FBitmap.Width := FDrawRect.Right;
    FBitmap.Height := FDrawRect.Bottom;
    FBitmap.Canvas.Font.Assign(FCanvas.Font);
end;

(* fXgN^ *)
destructor	TProgress.Destroy;
begin
    FBitmap.Free;
	inherited Destroy;
end;

(* ` *)
procedure	TProgress.Draw();
var
	Ratio: double;
    Length: integer;
    Text: string;
    OffsetX, OffsetY: integer;
begin
	//wiFœh
    FBitmap.Canvas.Brush.Color := FBackColor;
    CanvasFillRect(FBitmap.Canvas, FDrawRect);
	//OiFœh
    FBitmap.Canvas.Brush.Color := FForeColor;
    Ratio := FValue / FMaxValue;
    Length := Round(FDrawRect.Right * Ratio);
    CanvasFillRect(FBitmap.Canvas, Rect(0, 0, Length, FDrawRect.Bottom));
	//gœh
    FBitmap.Canvas.Brush.Color := clBlack;
    FBitmap.Canvas.FrameRect(FDrawRect);
	//Sɒl\
	Text := FloatToStr(FValue);
    OffsetX := (FDrawRect.Right - FBitmap.Canvas.TextWidth(Text)) div 2;
    OffsetY := (FDrawRect.Bottom - FBitmap.Canvas.TextHeight(Text)) div 2;
    CanvasTextOut(FBitmap.Canvas, OffsetX, OffsetY, Text);
	//ۂɕ\
    CanvasBitBlt(FBitmap.Canvas, FCanvas, FRealRect);
    InvalidateRect(FCanvas.Handle, @FRealRect, True);
end;

(* lύX *)
procedure	TProgress.SetValue(AValue: double);
begin
    FValue := AValue;
	if FValue > FMaxValue then FValue := FMaxValue;
    Draw();
end;
}
//------------------------------------------------------------------------------

(*  *)
function	FmtMem(N: Integer): string;
const
	KB = 1024;
    MB = KB * KB;
    GB = KB * KB * KB;
begin
	if N > 1024 * 1024 * 1024 then
		Result := Format('%.1fGB', [N / GB])
	else
    if N > 1024 * 1024 then
		Result := Format('%.1fMB', [N / MB])
	else
		Result := Format('%.1fKB', [N / KB]);
end;
//--------------------------------------------
procedure TAboutBox.FormShow(Sender: TObject);
var
    MemInfo : TMemoryStatus;
begin
    MemInfo.dwLength := Sizeof (MemInfo);
    GlobalMemoryStatus (MemInfo);
	with MemInfo do
    begin
        Label1.Caption := TitledString('PHYSICAL:', Format('%s/%s', [FmtMem(dwAvailPhys), FmtMem(dwTotalPhys)]), 26);
        Label2.Caption := TitledString('PAGEFILE:', Format('%s/%s', [FmtMem(dwAvailPageFile), FmtMem(dwTotalPageFile)]), 26);
//        Label3.Caption := TitledString('VIRTUAL :', Format('%s/%s', [FmtMem(dwAvailVirtual), FmtMem(dwTotalVirtual)]), 26);
//        Label4.Caption := TitledString('ALLOCMEMCOUNT:', AllocMemCount, 26);
        Label5.Caption := TitledString('ALLOCMEMSIZE:', AllocMemSize, 26);
    end;
end;

(* z[y[WWv *)
procedure TAboutBox.LblURLClick(Sender: TObject);          
begin
    ExecuteFile(LblURL.Caption);
end;

(* EXEACRǂݍ *)
procedure TAboutBox.FormCreate(Sender: TObject);
begin
//    imgIcon.Picture.Assign(Application.Icon);
    lblCopy.Caption := ChangeFileExt(ExtractFileName(Application.ExeName), '');
end;

(* [ESC]I *)
procedure TAboutBox.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
	if (Key = VK_ESCAPE) then
    begin
    	Self.Close;
    end;
end;

end.

