unit iGraph;

interface

uses
	Windows, Forms, Graphics, Grids, Types;

(*----------------------------------------------------------------------------*)
function	CreateBitmap(AAssignCanvas: TCanvas): TBitmap;
function	CreateDib24Bitmap(W, H: integer): TBitmap;
(*----------------------------------------------------------------------------*)
procedure	CanvasBrushColor(ACanvas: TCanvas; State: TOwnerDrawState); overload;
procedure	CanvasBrushColor(ACanvas: TCanvas; State: TGridDrawState);  overload;
procedure	CanvasFillRect(ACanvas: TCanvas; ARect: TRect);
procedure	CanvasTextOut(ACanvas: TCanvas; AX, AY: integer; AText: string);
procedure	CanvasBitBlt(ASrc, ADest: TCanvas; ARect: TRect; ALeft: integer = 0; ATop: integer = 0);
procedure	CanvasDrawLine(ACanvas: TCanvas; AStart, AEnd: TPoint);
(*----------------------------------------------------------------------------*)
function	Color2Gray(AColor: TColor): TColor;
(*----------------------------------------------------------------------------*)
procedure	DrawPoint(ACanvas: TCanvas; AX, AY: double; ARadius: integer = 3;
						ABrushColor: TColor = clBlack; APenColor: TColor = clBlack);
procedure	DrawLine(ACanvas: TCanvas; AX1, AY1, AX2, AY2: double);
procedure	DrawText(ACanvas: TCanvas; AX, AY: double; AText: string);
procedure	DrawArrow(ACanvas: TCanvas; AX1, AY1, AX2, AY2: double);
(*----------------------------------------------------------------------------*)
function	RectWidth(ARect: TRect): integer;
function	RectHeight(ARect: TRect): integer;

implementation

uses
	SysUtils;

(*
	=====================================================================
	TBitmap쐬
	=====================================================================
*)

(* rbg}bvIuWFNg쐬 *)
function	CreateBitmap(AAssignCanvas: TCanvas): TBitmap;
begin
	Result := TBitmap.Create;
    Result.Width := Screen.Width;
    Result.Height := Screen.Height;
    Result.Canvas.Font.Assign(AAssignCanvas.Font);
end;

(* 24bit DIB ۂ̂ *)
function	CreateDib24Bitmap(W, H: integer): TBitmap;
begin
    Result := TBitmap.Create;
    Result.Width := W;
    Result.Height := H;
    Result.HandleType := bmDIB;
    Result.PixelFormat := pf24bit;
end;

(*
	=====================================================================
	I[i[h[p
	=====================================================================
*)

(* uVF *)
procedure	CanvasBrushColor(ACanvas: TCanvas; State: TOwnerDrawState);
begin
	if (odSelected in State) then
    begin
    	ACanvas.Brush.Color := RGB(204, 204, 255);
    end else
    begin
    	ACanvas.Brush.Color := RGB(240, 240, 240);
    end;
end;

(* uVF *)
procedure	CanvasBrushColor(ACanvas: TCanvas; State: TGridDrawState);
begin
	if (gdSelected in State) then
    begin
    	ACanvas.Brush.Color := RGB(204, 204, 255);
    end else
    begin
    	ACanvas.Brush.Color := RGB(240, 240, 240);
    end;
end;

(* hԂ *)
procedure	CanvasFillRect(ACanvas: TCanvas; ARect: TRect);
begin
	ACanvas.FillRect(Rect(0, 0, ARect.Right - ARect.Left, ARect.Bottom - ARect.Top));
end;

(* ` *)
procedure	CanvasTextOut(ACanvas: TCanvas; AX, AY: integer; AText: string);
var
    OldBkMode: integer;
begin
    OldBkMode := SetBkMode(ACanvas.Handle, Integer(TRANSPARENT));
    ACanvas.TextOut(AX, AY, AText);
    SetBkMode(ACanvas.Handle, OldBkMode);
end;

(* ` *)
procedure	CanvasBitBlt(ASrc, ADest: TCanvas; ARect: TRect; ALeft: integer = 0; ATop: integer = 0);
begin
    BitBlt(
        ADest.Handle,
        ARect.Left + ALeft,
        ARect.Top + ATop,
        ARect.Right - ARect.Left,
        ARect.Bottom - ARect.Top,
        ASrc.Handle, 0, 0, SRCCOPY
    );
end;

(*  *)
procedure	CanvasDrawLine(ACanvas: TCanvas; AStart, AEnd: TPoint);
begin
	ACanvas.MoveTo(AStart.X, AStart.Y);
	ACanvas.LineTo(AEnd.X, AEnd.Y);
end;

(*
	=====================================================================
	ϊ֐
	=====================================================================
*)

(* O[XP[ *)
function	Color2Gray(AColor: TColor): TColor;
var
	R, G, B: double;
begin
	R := GetRValue(AColor);
	G := GetGValue(AColor);
	B := GetBValue(AColor);
    Result := RGB(Round(0.299 * R), Round(0.587 * G), Round(0.114 * B));
end;

(*
	=====================================================================
	{}`
	=====================================================================
*)

(* _ *)
procedure	DrawPoint(ACanvas: TCanvas; AX, AY: double; ARadius: integer = 3;
						ABrushColor: TColor = clBlack; APenColor: TColor = clBlack);
var
    Left, Top, Bottom, Right: integer;
    OldPen, OldBrush: TColor;
begin
    OldPen   := ACanvas.Pen.Color;
    OldBrush := ACanvas.Brush.Color;

    Left   := Round(AX) - ARadius;
    Top    := Round(AY) - ARadius;
    Right  := Round(AX) + ARadius;
    Bottom := Round(AY) + ARadius;

    ACanvas.Pen.Color	:= APenColor;
    ACanvas.Brush.Color := ABrushColor;

	ACanvas.Ellipse(Left, Top, Right, Bottom);

    ACanvas.Pen.Color	:= OldPen;
    ACanvas.Brush.Color := OldBrush;
end;

(*  *)
procedure	DrawLine(ACanvas: TCanvas; AX1, AY1, AX2, AY2: double);
begin
    ACanvas.MoveTo(Round(AX1), Round(AY1));
    ACanvas.LineTo(Round(AX2), Round(AY2));
end;

(*  *)
procedure	DrawText(ACanvas: TCanvas; AX, AY: double; AText: string);
var
    OldBkMode:Integer;
begin
    OldBkMode := SetBkMode(ACanvas.Handle, Integer(TRANSPARENT));
    ACanvas.TextOut(Round(AX), Round(AY), AText);
    SetBkMode(ACanvas.Handle, OldBkMode);
end;

(*  *)
procedure	DrawArrow(ACanvas: TCanvas; AX1, AY1, AX2, AY2: double);
var
	OldPenColor: TColor;
    OldPenWidth: integer;
begin
    OldPenColor := ACanvas.Pen.Color;
    OldPenWidth := ACanvas.Pen.Width;
    ACanvas.Pen.Color := clRed;
    ACanvas.Pen.Width := 2;
	DrawLine(ACanvas, AX1, AY1, AX2, AY2);
    ACanvas.Pen.Color := OldPenColor;
    ACanvas.Pen.Width := OldPenWidth;
end;

(* TRect  ƍ *)
function	RectWidth(ARect: TRect): integer;
begin
	Result := ARect.Right - ARect.Left;
end;

function	RectHeight(ARect: TRect): integer;
begin
	Result := ARect.Bottom - ARect.Top;
end;

end.
