unit iUtils;

interface

uses
	Windows, Classes, Graphics, Types;

(*----------------------------------------------------------------------------*)
function	AddHeadChar(const AStr: string; const AChar: char; const ALen: integer): string;
function	AddTailChar(const AStr: string; const AChar: char; const ALen: integer): string;
function	SandStr(const AStr: string; const AHead, ATail: Char): string;
function	SandStrSqrBlacket(const AStr: string): string;
function	SandStrBlacket(const AStr: string): string;
function	HTTPEncode(const AStr: String): String;
type		TTextAlign = (taLeft = 0, taLMiddle = 1, taRMiddle = 2, taRight = 3);
function	TitledString(ATitle: string; AValue: string; ALen: integer; AAlign: TTextAlign = taRight): string; overload;
function	TitledString(ATitle: string; AValue: integer; ALen: integer; AAlign: TTextAlign = taRight): string; overload;
function	TitledString(ATitle: string; AValue: double; ALen: integer; AAlign: TTextAlign = taRight): string; overload;
function	TitledString(ATitle: string; AValue: Boolean; ALen: integer; AAlign: TTextAlign = taRight): string; overload;
(*----------------------------------------------------------------------------*)
function	IsFolderPath(APath: string): Boolean;
procedure   LoadFromFile(APath: string; AStrings: TStrings; IgnoreZeroLine: Boolean = False);
function	ExecuteFile(FilePath: string): Boolean;
function	CheckExtension(AFileName, AExtension: string): Boolean;
(*----------------------------------------------------------------------------*)
procedure	PutLog(AStr: string);	overload;
procedure	PutLog(AStr: integer);	overload;
procedure	PutLog(AStr: double);	overload;
procedure	PutLogEx(AStr: string);		overload;
procedure	PutLogEx(AStr: integer);	overload;
procedure	PutLogEx(AStr: double);		overload;
(*----------------------------------------------------------------------------*)
function	IsSingleByte(AStr: string; AByteIndex: integer): Boolean;
function	NotSingleByte(AStr: string; AByteIndex: integer): Boolean;
function	IsLeadByte(AStr: string; AByteIndex: integer): Boolean;
function	IsTrailByte(AStr: string; AByteIndex: integer): Boolean;
(*----------------------------------------------------------------------------*)
function	isalnum(c: char): Boolean;
function	isalpha(c: char): Boolean;
function	iscntrl(c: char): Boolean;
function	isdigit(c: char): Boolean;
function	isgraph(c: char): Boolean;
function	islower(c: char): Boolean;
function	isprint(c: char): Boolean;
function	ispunct(c: char): Boolean;
function	isspace(c: char): Boolean;
function	isupper(c: char): Boolean;
function	isxdigit(c: char): Boolean;
(*----------------------------------------------------------------------------*)
function	IsInteger(Text: string): Boolean;
function	IsDouble(Text: string): Boolean;
(*----------------------------------------------------------------------------*)
function	InputBoxInt(const ACaption, APrompt: string; const ADefault: integer): integer;
function	InputBoxDbl(const ACaption, APrompt: string; const ADefault: double): double;
(*----------------------------------------------------------------------------*)
function	ShowConfirmation(const Msg: string; HelpCtx: integer = 0): Boolean;
procedure	MessageBox(const Text: string);		overload;
procedure	MessageBox(const Number: integer);  overload;
procedure	MessageBox(const Number: double);	overload;
procedure	ShowError(const Msg: string; HelpCtx: integer = 0);
procedure	ShowInformation(const Msg: string; HelpCtx: integer = 0);
procedure	ShowWarning(const Msg: string; HelpCtx: integer = 0);
(*----------------------------------------------------------------------------*)
function	GetOsVersion(): string;
function	AppExeName(): string;
function	AppExePath(): string;
procedure	GetScreenFonts(pResult: TStrings);
(*----------------------------------------------------------------------------*)
procedure	SendToRecycleBin(FileName: string);
(*----------------------------------------------------------------------------*)
function	PointToStr(const APoint: TPoint): string;
function	NowToStr(): string;
function	TabToSpace(AText: string; ASpaceCount: integer = 2): string;
(*----------------------------------------------------------------------------*)

var
	AppLogName: string;
	AppDatName: string;

implementation

uses
    ShellApi, Forms, SysUtils, Dialogs, Controls;

(*
	=====================================================================
	񑀍֐
	=====================================================================
*)

(* ̑OɎw肵oCgɂȂ܂ŕǉ *)
function	AddHeadChar(const AStr: string; const AChar: char; const ALen: integer): string;
begin
	Result := AStr;
	while (Length(Result) < ALen) do Result := AChar + Result;
end;

(* ̌Ɏw肵oCgɂȂ܂ŕǉ *)
function	AddTailChar(const AStr: string; const AChar: char; const ALen: integer): string;
begin
	Result := AStr;
	while (Length(Result) < ALen) do Result := Result + AChar;
end;

(* w肵ŋ0 *)
function	SandStr(const AStr: string; const AHead, ATail: Char): string;
begin
	Result := AHead + AStr + ATail;
end;
(* w肵ŋ1 *)
function	SandStrSqrBlacket(const AStr: string): string;
begin
	Result := SandStr(AStr, '[', ']');
end;
(* w肵ŋ2 *)
function	SandStrBlacket(const AStr: string): string;
begin
	Result := SandStr(AStr, '(', ')');
end;
(* http://fobec.free.fr/SQL_truc/lecture_truc.php3?num=347 *)
(* HTTPEncode *)
function	HTTPEncode(const AStr: String): String;
const
    NoConversion = ['A'..'Z','a'..'z','*','@','.','_','-'];
var
    Sp, Rp: PChar;
begin
    SetLength(Result, Length(AStr) * 3);
    Sp := PChar(AStr);
    Rp := PChar(Result);
    while Sp^ <> #0 do
    begin
        if Sp^ in NoConversion then
            Rp^ := Sp^
        else
        if Sp^ = ' ' then
            Rp^ := '+'
        else
        begin
            FormatBuf(Rp^, 3, '%%%.2x', 6, [Ord(Sp^)]);
            Inc(Rp,2);
        end;
        Inc(Rp);
        Inc(Sp);
    end;
    SetLength(Result, Rp - PChar(Result));
end;
(* Ƀ^CgŜw肵ɂȂ悤ɔpXy[Xt *)
function	TitledString(ATitle, AValue: string; ALen: integer; AAlign: TTextAlign = taRight): string;
var
	Len: integer;
    FullSpace, ShortSpace, LongSpace: string;
begin
    Len := Length(ATitle + AValue);
    if Len >= ALen then
    begin
    	Result := ATitle + AValue;
    end else
    begin
		ShortSpace := StringOfChar(' ', (ALen - Len) div 2);
        LongSpace  := StringOfChar(' ', ALen - Len - Length(ShortSpace));
        FullSpace  := ShortSpace + LongSpace;
        case AAlign of
        taLeft:
        	Result := ATitle + AValue + FullSpace;
        taLMiddle:
        	Result := ATitle + ShortSpace + AValue + LongSpace;
        taRMiddle:
        	Result := ATitle + LongSpace + AValue + ShortSpace;
        taRight:
        	Result := ATitle + FullSpace + AValue;
        else
        	Result := '';
        end;
    end;
end;

function	TitledString(ATitle: string; AValue: integer; ALen: integer; AAlign: TTextAlign = taRight): string;
begin
    Result := TitledString(ATitle, IntToStr(AValue), ALen, AAlign);
end;

function	TitledString(ATitle: string; AValue: double; ALen: integer; AAlign: TTextAlign = taRight): string;
begin
    Result := TitledString(ATitle, FloatToStr(AValue), ALen, AAlign);
end;

function	TitledString(ATitle: string; AValue: Boolean; ALen: integer; AAlign: TTextAlign = taRight): string;
begin
    Result := TitledString(ATitle, BoolToStr(AValue, True), ALen, AAlign);
end;

(*
	=====================================================================
	t@C֐
	=====================================================================
*)

(* pXtH_t@C𒲂ׂ *)
function IsFolderPath(APath: string): Boolean;
var
	res: Boolean;
begin
	Result := False;
    res := DirectoryExists(APath);
    if res then
    begin
    	Result := True;
    end else
    begin
		res := FileExists(APath);
        if (not res) then
        begin
			raise Exception.Create('is not path');
        end;
    end;
end;

(* t@C̓ǂݍ *)
procedure   LoadFromFile(APath: string; AStrings: TStrings; IgnoreZeroLine: Boolean = False);
var
    F: TextFile;
    S: string;
begin
    AStrings.Clear;
	AssignFile(F, APath);
    Reset(F);
    while not EOF(F) do begin
        Readln(F, S);
        if IgnoreZeroLine and (Length(Trim(S)) = 0) then Continue;
        AStrings.Add(S);
    end;
    CloseFile(F);
end;

(*	ShellExecutẽbp[֐	*)
function ExecuteFile(FilePath: string): Boolean;
const
    ERROR_NO = 32;
var
    Value: integer;
begin
    Value := ShellExecute(
                    Application.MainForm.handle,
                    'open',
                    PChar(FilePath),
                    '',
                    PChar(ExtractFilePath(Application.Exename)),
                    SW_SHOWDEFAULT
                    );
    Result := (Value > ERROR_NO);
end;

(* gq`FbN *)
function	CheckExtension(AFileName, AExtension: string): Boolean;
begin
	Result := (UpperCase(ExtractFileExt(AFileName)) = UpperCase(AExtension));
end;

(* Ot@C쐬 *)
procedure	PutLog(AStr: string);
var
	F: TextFile;
begin
	if AStr = '' then Exit;
    AssignFile(F, AppLogName);
	try
        if FileExists(AppLogName) then Append(F) else Rewrite(F);
        Writeln(f, AStr);
        Flush(F);
	finally
	    CloseFile(F);
    end;
end;

procedure	PutLog(AStr: integer);
begin
	PutLog(IntToStr(AStr));
end;

procedure	PutLog(AStr: double);
begin
	PutLog(FloatToStr(AStr));
end;

procedure	PutLogEx(AStr: string);
begin
	if AStr = '' then Exit;
	PutLog(FormatDateTime('YYYY/MM/DD HH:NN:SS', Now) + ' >> ' + AStr);
end;

procedure	PutLogEx(AStr: integer);
begin
	PutLogEx(IntToStr(AStr));
end;

procedure	PutLogEx(AStr: double);
begin
	PutLogEx(FloatToStr(AStr));
end;

(*
	=====================================================================
	֐
	=====================================================================
*)
(* S 1  *)
function IsSingleByte(AStr: string; AByteIndex: integer): Boolean;
begin
	Result := (StrByteType(PChar(AStr), AByteIndex) = mbSingleByte);
end;

(* 2 oCg̈ꕔ *)
function NotSingleByte(AStr: string; AByteIndex: integer): Boolean;
begin
	Result := (StrByteType(PChar(AStr), AByteIndex) <> mbSingleByte);
end;

(* 2 oCg̍ŏ̃oCg *)
function IsLeadByte(AStr: string; AByteIndex: integer): Boolean;
begin
	Result := (StrByteType(PChar(AStr), AByteIndex) = mbLeadByte);
end;

(* 2 oCg 2 Ԗڂ̃oCg *)
function IsTrailByte(AStr: string; AByteIndex: integer): Boolean;
begin
	Result := (StrByteType(PChar(AStr), AByteIndex) = mbTrailByte);
end;

(*
	=====================================================================
	Cꕗ ֐
	=====================================================================
*)

(* 啶`FbN *)
function	isupper(c: char): Boolean;
begin
	Result := (c in ['A'..'Z']);
end;

(* q`FbN *)
function	islower(c: char): Boolean;
begin
	Result := (c in ['a'..'z']);
end;

(* p`FbN *)
function	isalpha(c: char): Boolean;
begin
	Result := isupper(c) or islower(c);
end;

(* `FbN *)
function	isdigit(c: char): Boolean;
begin
	Result := (c in ['0'..'9']);
end;

(* p`FbN *)
function	isalnum(c: char): Boolean;
begin
	Result := isalpha(c) or isdigit(c);
end;

(* 16i`FbN *)
function	isxdigit(c: char): Boolean;
begin
	Result := isdigit(c) or (c in ['a'..'f', 'A'..'F']);
end;

(* 䕶`FbN *)
function	iscntrl(c: char): Boolean;
begin
	Result := (c in [#$00..#$1F, #$7F]);
end;

(* 󔒌n`FbN *)
function	isspace(c: char): Boolean;
begin
	Result := (c in [#$09..#$0D, #$20]);
end;

(* 󎚉\`FbN *)
function	isprint(c: char): Boolean;
begin
	Result := (c in [#$20..#$7E]);
end;

(* 󔒈ȊÖ󎚉\`FbN *)
function	isgraph(c: char): Boolean;
begin
	Result := (c in [#$21..#$7E]);
end;

(* ؂蕶`FbN *)
function	ispunct(c: char): Boolean;
begin
    Result := isgraph(c) and (not isalnum(c));
end;

(*
	=====================================================================
	`FbN֐
	=====================================================================
*)

(* l`FbN *)
function	IsNumber(Text: string; var Number: double): Boolean;
var
	Res: double;
    Err: Integer;
begin
    Val(Text, Res, Err);
	Result := (Err = 0);
    if Result then Number := Res;
end;

(* `FbN *)
function	IsInteger(Text: string): Boolean;
var
	Number: double;
begin
	if IsNumber(Text, Number) then
    	Result := (Pos('.', Text) = 0)
    else
    	Result := False;
end;

(* `FbN Integer̍ől͏Ɏs *)
function	IsDouble(Text: string): Boolean;
var
	Number: double;
begin
	Result := IsNumber(Text, Number);
end;

(*
	=====================================================================
	_CAO֐
	=====================================================================
*)

(* l *)
function	InputBoxInt(const ACaption, APrompt: string; const ADefault: integer): integer;
const
	ERR_NOT_NUM = 'l͂ĂB';
var
	Value, Prompt: string;
begin
    Value := '';
    while True do
    begin
		Prompt := APrompt;
        if Value <> '' then Prompt := Prompt + ERR_NOT_NUM;
        Value := InputBox(ACaption, Prompt, IntToStr(ADefault));
    	if IsInteger(Value) then Break;
    end;
    Result := StrToInt(Value);
end;

(* l *)
function	InputBoxDbl(const ACaption, APrompt: string; const ADefault: double): double;
const
	ERR_NOT_NUM = 'l͂ĂB';
var
	Value, Prompt: string;
begin
    Value := '';
    while True do
    begin
		Prompt := APrompt;
        if Value <> '' then Prompt := Prompt + ERR_NOT_NUM;
        Value := InputBox(ACaption, Prompt, FloatToStr(ADefault));
    	if IsDouble(Value) then Break;
    end;
    Result := StrToFloat(Value);
end;

(* xbZ[W\ *)
procedure	ShowWarning(const Msg: string; HelpCtx: integer = 0);
begin
	MessageDlg(Msg, mtWarning, [mbOK], HelpCtx);
end;

(* G[bZ[W\ *)
procedure	ShowError(const Msg: string; HelpCtx: integer = 0);
begin
	MessageDlg(Msg, mtError, [mbOK], HelpCtx);
end;

(* 񃁃bZ[W\ *)
procedure	ShowInformation(const Msg: string; HelpCtx: integer = 0);
begin
	MessageDlg(Msg, mtInformation, [mbOK], HelpCtx);
end;

(* ⃁bZ[W\ *)
function	ShowConfirmation(const Msg: string; HelpCtx: integer = 0): Boolean;
begin
	Result := (MessageDlg(Msg, mtConfirmation, [mbYes, mbNo], HelpCtx) = mrYes);
end;

(* bZ[W\ *)
procedure	MessageBox(const Text: string);
begin
	Application.MessageBox(PChar(Text), PChar(Application.Title));
end;

(* bZ[W\2 *)
procedure	MessageBox(const Number: integer);
begin
    MessageBox(IntToStr(Number));
end;

(* bZ[W\3 *)
procedure	MessageBox(const Number: double);
begin
    MessageBox(FloatToStr(Number));
end;

(*
	=====================================================================
	VXe
	=====================================================================
*)

(* OS o[W << K *)
function	GetOsVersion(): string;

	function	PlatformId(ID: Cardinal): string;
    begin
		case ID of
            VER_PLATFORM_WIN32s			: Result := 'Win32s on Windows 3.1';
            VER_PLATFORM_WIN32_WINDOWS	: Result := 'Win32 on Windows 95';
            VER_PLATFORM_WIN32_NT		: Result := 'Win32 on Windows NT';
        end;
    end;

var
	osi: TOSVersionInfo;
begin
	FillChar(osi, SizeOf(TOSVersionInfo), #0);
    osi.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
    GetVersionEx(osi);
    Result := Format('%s Version %d.%d (Build %d : %s)',
    				[PlatformId(osi.dwPlatformId),
                     osi.dwMajorVersion,
                     osi.dwMinorVersion,
                     osi.dwBuildNumber,
                     osi.szCSDVersion
                    ]);
end;

(* AvP[Vt@C *)
function	AppExeName(): string;
begin
	Result := ChangeFileExt(ExtractFileName(Application.ExeName), '');
end;

(* AvP[Vt@C *)
function	AppExePath(): string;
begin
	Result := ExtractFilePath(Application.ExeName);
end;

(* XN[tHg *)
procedure	GetScreenFonts(pResult: TStrings);
var
    i: integer;
begin
    //ScreenϐoB
    pResult.Assign(Screen.Fonts);
    //cp(?)tHgĈłǂB
    for i := pResult.Count - 1 downto 0 do
    begin
        if Copy(pResult[i], 1, 1) = '@' then
        begin
            pResult.Delete(i);
        end;
    end;
end;

(*
	=====================================================================
	SHFileOperation
	=====================================================================
*)

(* ݔs *)
procedure	SendToRecycleBin(FileName: string);
var
	SHF: TSHFileOpStruct;
begin
    with SHF do
    begin
        Wnd		:= Application.Handle;
        wFunc	:= FO_DELETE;
        pFrom	:= PChar(FileName + #0);
        fFlags	:= FOF_SILENT or FOF_ALLOWUNDO;
    end;
    SHFileOperation(SHF);
end;

(*
	=====================================================================
	̑
	=====================================================================
*)

(* |Cg\ *)
function	PointToStr(const APoint: TPoint): string;
begin
	Result := Format('X = %4d , Y = %4d', [APoint.X, APoint.Y]);
end;

function	NowToStr(): string;
begin
	Result := FormatDateTime('YYYY/MM/DD HH:NN:SS', Now);
end;

(* Tab -> Space ϊ *)
function	TabToSpace(AText: string; ASpaceCount: integer = 2): string;
begin
	Result := StringReplace(AText, #9, StringOfChar(' ', ASpaceCount), [rfReplaceAll]); 
end;

(*
	=====================================================================
	
	=====================================================================
*)

initialization
	AppLogName := ChangeFileExt(Application.ExeName, '.log');
	AppDatName := ChangeFileExt(Application.ExeName, '.dat');

	//
	//DEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUG







	//DEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUG
	//

end.
